﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    /// <summary>
    /// Duplicates the top item on the INTEGER stack. Does not pop its argument
    /// (which, if it did, would negate the effect of the duplication!).
    /// </summary>
    /// <typeparam name="T">Stacktype</typeparam>
    public abstract class DuplicateExpression<T> : Expression
    {
        public DuplicateExpression(string stringRepresentation) : base(stringRepresentation)
        { }

        public void Eval(IStack<T> stack)
        {
            if (stack.Count == 0)
                return;

            stack.Push(stack.Top);
        }
    }

    public class IntegerDuplicateExpression : DuplicateExpression<long>
    {
        public IntegerDuplicateExpression() : base("INTEGER.DUP")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.IntegerStack);
        }
    }

    public class FloatDuplicateExpression : DuplicateExpression<double>
    {
        public FloatDuplicateExpression() : base("FLOAT.DUP")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.FloatStack);
        }
    }

    public class BooleanDuplicateExpression : DuplicateExpression<bool>
    {
        public BooleanDuplicateExpression() : base("BOOLEAN.DUP")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.BooleanStack);
        }
    }

    public class NameDuplicateExpression : DuplicateExpression<string>
    {
        public NameDuplicateExpression() : base("NAME.DUP")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.NameStack);
        }
    }

    public class ExecDuplicateExpression : DuplicateExpression<Expression>
    {
        public ExecDuplicateExpression() : base("EXEC.DUP")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.ExecStack);
        }
    }

    public class CodeDuplicateExpression : DuplicateExpression<Expression>
    {
        public CodeDuplicateExpression() : base("CODE.DUP")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.CodeStack);
        }
    }
}
