﻿using System.Linq;
using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Core;
using HeuristicLab.Random;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    public class NameDefineXExecExpression : Expression
    {
        public NameDefineXExecExpression(string name) : base(name) { }

        public override void Eval(IInterpreter interpreter)
        {
            Expression expression;
            if (!interpreter.IsNameQuoteFlagSet &&
                interpreter.CustomExpressions.TryGetValue(this.StringRepresentation, out expression))
            {
                interpreter.ExecStack.Push(expression);
            }
            else
            {
                interpreter.NameStack.Push(this.StringRepresentation);
                interpreter.IsNameQuoteFlagSet = false;
            }
        }

        public override bool Equals(object obj)
        {
            if (!base.Equals(obj))
            {
                return false;
            }

            return this.StringRepresentation.Equals(obj.ToString());
        }
    }

    /// <summary>
    /// Sets a flag indicating that the next name encountered will be pushed onto the NAME stack (and not have its associated value
    /// pushed onto the EXEC stack), regardless of whether or not it has a definition. Upon encountering such a name and pushing it
    /// onto the NAME stack the flag will be cleared (whether or not the pushed name had a definition).
    /// </summary>
    public class NameQuoteExpression : Expression
    {
        public NameQuoteExpression() : base("NAME.QUOTE") { }

        public override void Eval(IInterpreter interpreter)
        {
            interpreter.IsNameQuoteFlagSet = true;
        }
    }

    /// <summary>
    /// Pushes a randomly selected NAME that already has a definition.
    /// </summary>
    public class NameRandBoundNameExpression : Expression
    {
        private static IRandom rand = new FastRandom();

        public NameRandBoundNameExpression() : base("NAME.RANDBOUNDNAME") { }

        public override void Eval(IInterpreter interpreter)
        {
            if (!interpreter.CustomExpressions.Any())
                return;

            var index = rand.Next(0, interpreter.CustomExpressions.Keys.Count - 1);
            var name = interpreter.CustomExpressions.Keys.ElementAt(index);

            interpreter.NameStack.Push(name);
        }
    }
}
