﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    public abstract class PopExpression<T> : Expression
    {
        public PopExpression(string stringRepresentation) : base(stringRepresentation)
        {
        }

        public void Eval(IStack<T> stack)
        {
            if (stack.Count == 0)
                return;

            stack.Pop();
        }
    }

    public class IntegerPopExpression : PopExpression<long>
    {
        public IntegerPopExpression() : base("INTEGER.POP")
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.IntegerStack);
        }
    }

    public class FloatPopExpression : PopExpression<double>
    {
        public FloatPopExpression() : base("FLOAT.POP")
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.FloatStack);
        }
    }

    public class BooleanPopExpression : PopExpression<bool>
    {
        public BooleanPopExpression() : base("BOOLEAN.POP")
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.BooleanStack);
        }
    }

    public class NamePopExpression : PopExpression<string>
    {
        public NamePopExpression() : base("NAME.POP")
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.NameStack);
        }
    }

    public class ExecPopExpression : PopExpression<Expression>
    {
        public ExecPopExpression() : base("EXEC.POP")
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.ExecStack);
        }
    }

    public class CodePopExpression : PopExpression<Expression>
    {
        public CodePopExpression() : base("CODE.POP")
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.CodeStack);
        }
    }
}
