﻿using System;
using System.Collections.Generic;
using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    /// <summary>
    /// Binds the top of stack to the NAME on top of the NAME stack.
    /// </summary>
    /// <typeparam name="T">Stacktype</typeparam>
    public abstract class SetExpressions<T> : Expression
    {
        public SetExpressions(string stringRepresentation) : base(stringRepresentation)
        {
        }

        public void Eval(IStack<T> stack, IStack<string> nameStack, IDictionary<string, Expression> customExpressions, Func<T, Expression> creator)
        {
            if (stack.Count == 0 || nameStack.Count == 0)
                return;

            var name = nameStack.Pop();
            var value = stack.Pop();
            var expression = creator(value);

            if (customExpressions.ContainsKey(name))
            {
                customExpressions[name] = expression;
            }
            else
            {
                customExpressions.Add(name, expression);
            }
        }
    }

    public class IntegerSetExpression : SetExpressions<long>
    {
        public IntegerSetExpression() : base("INTEGER.SET")
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(
                interpreter.IntegerStack,
                interpreter.NameStack,
                interpreter.CustomExpressions,
                v => new IntegerPushExpression(v));
        }
    }

    public class FloatSetExpression : SetExpressions<double>
    {
        public FloatSetExpression() : base("FLOAT.SET")
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(
                interpreter.FloatStack,
                interpreter.NameStack,
                interpreter.CustomExpressions,
                v => new FloatPushExpression(v));
        }
    }
}
