﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    public abstract class ShoveExpression<T> : Expression
    {
        public ShoveExpression(string stringRepresentation) : base(stringRepresentation)
        { }

        public void Eval(IStack<T> stack, IStack<long> integerStack, int count = 1)
        {
            if (integerStack.Count == 0)
                return;

            if (integerStack.Top > stack.Count - count ||
                integerStack.Top < 0 ||
                stack.Count < 2)
                return;

            var index = integerStack.Pop();
            var item = stack.Pop();

            // TODO: What if index > int.MaxValue ?
            stack.Insert((int)index, item);
        }
    }

    public class IntegerShoveExpression : ShoveExpression<long>
    {
        public IntegerShoveExpression() : base("INTEGER.SHOVE")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.IntegerStack, interpreter.IntegerStack, 2);
        }
    }

    public class FloatShoveExpression : ShoveExpression<double>
    {
        public FloatShoveExpression() : base("FLOAT.SHOVE")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.FloatStack, interpreter.IntegerStack);
        }
    }

    public class BooleanShoveExpression : ShoveExpression<bool>
    {
        public BooleanShoveExpression() : base("BOOLEAN.SHOVE")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.BooleanStack, interpreter.IntegerStack);
        }
    }

    public class NameShoveExpression : ShoveExpression<string>
    {
        public NameShoveExpression() : base("NAME.SHOVE")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.NameStack, interpreter.IntegerStack);
        }
    }

    public class ExecShoveExpression : ShoveExpression<Expression>
    {
        public ExecShoveExpression() : base("EXEC.SHOVE")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.ExecStack, interpreter.IntegerStack);
        }
    }

    public class CodeShoveExpression : ShoveExpression<Expression>
    {
        public CodeShoveExpression() : base("CODE.SHOVE")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.CodeStack, interpreter.IntegerStack);
        }
    }
}
