﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    /// <summary>
    /// Pushes the stack depth onto the INTEGER stack (thereby increasing it!).
    /// </summary>
    /// <typeparam name="T">Stacktype</typeparam>
    public abstract class StackdepthExpression<T> : Expression
    {
        public StackdepthExpression(string stringRepresentation) : base(stringRepresentation)
        { }

        public void Eval(IStack<T> stack, IStack<long> integerStack, bool incremental = false)
        {
            var count = stack.Count;

            if (incremental)
            {
                count += 1;
            }

            integerStack.Push(count);
        }
    }

    public class IntegerStackdepthExpression : StackdepthExpression<long>
    {
        public IntegerStackdepthExpression() : base("INTEGER.STACKDEPTH")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.IntegerStack, interpreter.IntegerStack, true);
        }
    }

    public class FloatStackdepthExpression : StackdepthExpression<double>
    {
        public FloatStackdepthExpression() : base("FLOAT.STACKDEPTH")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.FloatStack, interpreter.IntegerStack);
        }
    }

    public class BooleanStackdepthExpression : StackdepthExpression<bool>
    {
        public BooleanStackdepthExpression() : base("BOOLEAN.STACKDEPTH")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.BooleanStack, interpreter.IntegerStack);
        }
    }

    public class NameStackdepthExpression : StackdepthExpression<string>
    {
        public NameStackdepthExpression() : base("NAME.STACKDEPTH")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.NameStack, interpreter.IntegerStack);
        }
    }

    public class ExecStackdepthExpression : StackdepthExpression<Expression>
    {
        public ExecStackdepthExpression() : base("EXEC.STACKDEPTH")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.ExecStack, interpreter.IntegerStack);
        }
    }

    public class CodeStackdepthExpression : StackdepthExpression<Expression>
    {
        public CodeStackdepthExpression() : base("CODE.STACKDEPTH")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.CodeStack, interpreter.IntegerStack);
        }
    }
}
