﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    /// <summary>
    /// Removes an indexed item from "deep" in the stack and pushes it on top of the stack.
    /// The index is taken from the INTEGER stack, and the indexing is done after the index is removed.
    /// </summary>
    /// <typeparam name="T">Stacktype</typeparam>
    public abstract class YankExpression<T> : Expression
    {
        public YankExpression(string stringRepresentation) : base(stringRepresentation)
        { }

        public void Eval(IStack<T> stack, IStack<long> integerStack, int count = 1)
        {
            if (integerStack.Count == 0)
                return;

            var index = integerStack.Top;

            if (index > stack.Count - count ||
                index < 0 ||
                stack.Count < 2)
                return;

            integerStack.Pop();
            stack.Yank((int)index);
        }
    }

    public class IntegerYankExpression : YankExpression<long>
    {
        public IntegerYankExpression() : base("INTEGER.YANK")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.IntegerStack, interpreter.IntegerStack, 2);
        }
    }

    public class FloatYankExpression : YankExpression<double>
    {
        public FloatYankExpression() : base("FLOAT.YANK")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.FloatStack, interpreter.IntegerStack);
        }
    }

    public class BooleanYankExpression : YankExpression<bool>
    {
        public BooleanYankExpression() : base("BOOLEAN.YANK")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.BooleanStack, interpreter.IntegerStack);
        }
    }

    public class NameYankExpression : YankExpression<string>
    {
        public NameYankExpression() : base("NAME.YANK")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.NameStack, interpreter.IntegerStack);
        }
    }

    public class ExecYankExpression : YankExpression<Expression>
    {
        public ExecYankExpression() : base("EXEC.YANK")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.ExecStack, interpreter.IntegerStack);
        }
    }

    public class CodeYankExpression : YankExpression<Expression>
    {
        public CodeYankExpression() : base("CODE.YANK")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.CodeStack, interpreter.IntegerStack);
        }
    }
}
