﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.BenchmarkSuite.Problems
{
    public class CountOdds : BenchmarkSuiteDataDescritpor<List<int>>
    {
        public const int MaxOdd = 999;
        public const int MaxEven = 1000;
        public const int MaxRandom = 1000;
        public const int MinRandom = 1000;
        public const int MaxVectorLength = 50;

        public override string Description { get { return String.Empty; } }

        public override string Name { get { return "Count Odds"; } }

        protected override IEnumerable<Example> GenerateExamples(IEnumerable<List<int>> inputData)
        {
            foreach (var input in inputData)
            {
                yield return new Example()
                {
                    Input = string.Format("[{0}]", string.Join(", ", input)),
                    Output = input.Count(x => x % 2 == 1).ToString()
                };
            }
        }

        protected override IEnumerable<List<int>> GenerateTest()
        {
            foreach (var list in this.GetHardcodedTrainingSamples())
                yield return list;

            foreach (var list in this.GetAllOdd(n: 100))
                yield return list;

            foreach (var list in this.GetAllEven(n: 100))
                yield return list;

            foreach (var list in this.GetRandom(n: 1800))
                yield return list;
        }

        protected override IEnumerable<List<int>> GenerateTraining()
        {
            foreach (var list in this.GetHardcodedTrainingSamples())
                yield return list;

            foreach (var list in this.GetAllOdd(n: 9))
                yield return list;

            foreach (var list in this.GetAllEven(n: 9))
                yield return list;

            foreach (var list in this.GetRandom(n: 150))
                yield return list;
        }

        private IEnumerable<List<int>> GetAllOdd(int n)
        {
            for (var i = 0; i < n; i++)
            {
                var length = rand.Next(0, MaxVectorLength);
                var vector = new List<int>(length);

                for (var j = 0; j < length; j++)
                {
                    vector.Add(rand.Next(0, MaxOdd) * 2 - MaxOdd);
                }

                yield return vector;
            }
        }

        private IEnumerable<List<int>> GetAllEven(int n)
        {
            for (var i = 0; i < n; i++)
            {
                var length = rand.Next(0, MaxVectorLength);
                var vector = new List<int>(length);

                for (var j = 0; j < length; j++)
                {
                    vector.Add(rand.Next(0, MaxEven) * 2 - MaxEven);
                }

                yield return vector;
            }
        }

        private IEnumerable<List<int>> GetRandom(int n)
        {
            for (var i = 0; i < n; i++)
            {
                var length = rand.Next(0, MaxVectorLength);
                var vector = new List<int>(length);

                for (var j = 0; j < length; j++)
                {
                    vector.Add(rand.Next(MinRandom, MaxRandom));
                }

                yield return vector;
            }
        }

        private IEnumerable<List<int>> GetHardcodedTrainingSamples()
        {
            yield return new List<int>() { };
            yield return new List<int>() { -10 };
            yield return new List<int>() { -9 };
            yield return new List<int>() { -8 };
            yield return new List<int>() { -7 };
            yield return new List<int>() { -6 };
            yield return new List<int>() { -5 };
            yield return new List<int>() { -4 };
            yield return new List<int>() { -3 };
            yield return new List<int>() { -2 };
            yield return new List<int>() { -1 };
            yield return new List<int>() { -0 };
            yield return new List<int>() { 1 };
            yield return new List<int>() { 2 };
            yield return new List<int>() { 3 };
            yield return new List<int>() { 4 };
            yield return new List<int>() { 5 };
            yield return new List<int>() { 6 };
            yield return new List<int>() { 7 };
            yield return new List<int>() { 8 };
            yield return new List<int>() { 9 };
            yield return new List<int>() { 10 };
            yield return new List<int>() { -947 };
            yield return new List<int>() { -450 };
            yield return new List<int>() { 303 };
            yield return new List<int>() { 886 };
            yield return new List<int>() { 0, 0 };
            yield return new List<int>() { 0, 1 };
            yield return new List<int>() { 7, 1 };
            yield return new List<int>() { -9, -1 };
            yield return new List<int>() { -11, 40 };
            yield return new List<int>() { 944, 77 };
        }
    }
}
