﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using HeuristicLab.Algorithms.PushGP.Expressions;
using HeuristicLab.Algorithms.PushGP.Interpreter;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace HeuristicLab.Tests.Benchmark.Problem
{
    [TestClass]
    public class CountOdds : Problem<long[], long>
    {
        public const string ProblemFileName = "examples-count-odds.csv";

        public CountOdds()
        {
            this.Examples = this.GetExamples(ProblemFileName, ExampleConverter.Integers, ExampleConverter.Integer);
        }

        [TestMethod]
        public void RandomWalk()
        {
            var limit = 1000;
            var iterations = 20000;
            var best = double.MaxValue;
            Expression bestProgram = null;
            var lockObj = new object();
            var globalExecCounter = 0;


            Parallel.For(0, iterations, async i =>
            {
                var interpreter = new PushGPInterpreter(new Configuration
                {
                    EvalPushLimit = 10000
                });

                var execCounter = 0;
                var program = interpreter.CodeGenerator.RandomProgram(limit);
                var results = new List<long>();

                foreach (var example in this.Examples)
                {
                    if (example.Input.Length > 0)
                    {
                        interpreter.IntegerStack.Push(example.Input);
                    }

                    await interpreter.InterpretAsync(program);

                    if (!interpreter.IntegerStack.IsEmpty)
                    {
                        var value = Math.Abs(example.Output - interpreter.IntegerStack.Top);

                        results.Add(Math.Min(value, 1000));
                    }

                    execCounter += interpreter.ExecCounter;
                    interpreter.Clear();
                }

                if (!results.Any())
                    return;

                var avg = results.Average();

                lock (lockObj)
                {
                    if (avg < best)
                    {
                        best = avg;
                        bestProgram = program;
                        globalExecCounter += execCounter;
                    }
                }
            });

            Debug.WriteLine("Best: {0}", best);
            Debug.WriteLine("Program: {0}", bestProgram);
            Debug.WriteLine("ExecCounter: {0}", globalExecCounter);
        }
    }
}
