﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace HeuristicLab.Tests.Interpreter.Expressions
{
    [TestClass]
    public class MixedExpressionTests : InterpreterTest
    {
        const double Delta = 0.00000001;

        [TestMethod]
        public void Example1()
        {
            this.interpreter.Interpret("( 2 3 INTEGER.* 4.1 5.2 FLOAT.+ TRUE FALSE BOOLEAN.OR )");

            Assert.AreEqual(6, interpreter.IntegerStack.Top);
            Assert.AreEqual(9.3, interpreter.FloatStack.Top);
            Assert.AreEqual(interpreter.BooleanStack.Top, true);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
        }

        [TestMethod]
        public void Example2()
        {
            this.interpreter.Interpret("( 5 1.23 INTEGER.+ ( 4 ) INTEGER.- 5.67 FLOAT.* )");

            Assert.AreEqual(1, interpreter.IntegerStack.Top);
            Assert.AreEqual(6.9741, interpreter.FloatStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
        }

        [TestMethod]
        public void Example3()
        {
            this.interpreter.Interpret("( 5 INTEGER.DUP INTEGER.+ )");

            Assert.AreEqual(10, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void Example4()
        {
            this.interpreter.Interpret("( 5 CODE.QUOTE ( INTEGER.DUP INTEGER.+ ) CODE.DO )");

            Assert.AreEqual(10, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void Example5()
        {
            this.interpreter.Interpret("( DOUBLE CODE.QUOTE ( INTEGER.DUP INTEGER.+ ) CODE.DEFINE 5 DOUBLE )");

            Assert.AreEqual(10, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void Example6()
        {
            this.interpreter.Interpret("( CODE.QUOTE ( INTEGER.DUP INTEGER.+ ) DOUBLE CODE.DEFINE 5 DOUBLE )");

            Assert.AreEqual(10, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void Example7()
        {
            this.interpreter.Interpret("( DOUBLE EXEC.DEFINE ( INTEGER.DUP INTEGER.+ ) 5 DOUBLE )");

            Assert.AreEqual(10, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void Example8()
        {
            this.interpreter.Configuration.TopLevelPushCode = true;

            this.interpreter.IntegerStack.Push(5);
            this.interpreter.Interpret(@"( CODE.QUOTE ( INTEGER.POP 1 )
                                           CODE.QUOTE ( CODE.DUP INTEGER.DUP 1 INTEGER.- CODE.DO INTEGER.* )
                                           INTEGER.DUP 2 INTEGER.< CODE.IF )");

            Assert.AreEqual(120, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.Count == 1);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void Example9()
        {
            this.interpreter.IntegerStack.Push(5);
            this.interpreter.Interpret("( 1 INTEGER.MAX 1 EXEC.DO*RANGE INTEGER.* )");

            Assert.AreEqual(120, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void Example10()
        {
            this.interpreter.IntegerStack.Push(5);
            this.interpreter.Interpret("( 1 INTEGER.MAX 1 EXEC.DO*RANGE ( 2 INTEGER.* ) )");

            Assert.AreEqual(2, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void Example11()
        {
            var program = PushGPInterpreter.Encode("( INTEGER.= CODE.QUOTE FLOAT.* CODE.QUOTE FLOAT./ CODE.IF )");

            this.interpreter.IntegerStack.Push(1, 1);
            this.interpreter.FloatStack.Push(2.1, 0.7);

            this.interpreter.Interpret(program);

            Assert.IsTrue(interpreter.IntegerStack.IsEmpty);
            Assert.AreEqual(1.47, interpreter.FloatStack.Top, Delta);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);

            this.interpreter.Clear();
            this.interpreter.IntegerStack.Push(1, 2);
            this.interpreter.FloatStack.Push(2.1, 0.7);
            this.interpreter.Interpret(program);

            Assert.IsTrue(interpreter.IntegerStack.IsEmpty);
            Assert.AreEqual(3d, interpreter.FloatStack.Top, Delta);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
        }

        [TestMethod]
        public void Example12()
        {
            var program = PushGPInterpreter.Encode("( INTEGER.= EXEC.IF FLOAT.* FLOAT./ )");

            this.interpreter.IntegerStack.Push(1, 1);
            this.interpreter.FloatStack.Push(2.1, 0.7);

            this.interpreter.Interpret(program);

            Assert.IsTrue(interpreter.IntegerStack.IsEmpty);
            Assert.AreEqual(1.47, interpreter.FloatStack.Top, Delta);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);

            this.interpreter.Clear();
            this.interpreter.IntegerStack.Push(1, 2);
            this.interpreter.FloatStack.Push(2.1, 0.7);
            this.interpreter.Interpret(program);

            Assert.IsTrue(interpreter.IntegerStack.IsEmpty);
            Assert.AreEqual(3d, interpreter.FloatStack.Top, Delta);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
        }

        [TestMethod]
        public void Example13()
        {
            this.interpreter.IntegerStack.Push(5);
            this.interpreter.Interpret("( EXEC.Y ( ( FALSE 2 INTEGER.* ) EXEC.IF ( ) EXEC.POP ) )");

            Assert.AreEqual(10, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void Example14()
        {
            this.interpreter.IntegerStack.Push(5);
            this.interpreter.Interpret("( EXEC.Y ( ( 2 INTEGER.* INTEGER.DUP 1000 INTEGER.< ) EXEC.IF ( ) EXEC.POP ) )");

            Assert.AreEqual(1280, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
            Assert.IsTrue(interpreter.FloatStack.IsEmpty);
        }

        [TestMethod]
        public void Example15()
        {
            this.interpreter.IntegerStack.Push(10);
            this.interpreter.FloatStack.Push(2);
            this.interpreter.Interpret(@"( ARG FLOAT.DEFINE
                                           EXEC.Y (
                                           ARG FLOAT.*
                                           1 INTEGER.-
                                           INTEGER.DUP 1 INTEGER.>
                                           EXEC.IF ( ) EXEC.POP
                                           )
                                         )");

            Assert.AreEqual(1024, interpreter.FloatStack.Top, Delta);
            Assert.AreEqual(1, interpreter.IntegerStack.Top);
            Assert.IsTrue(interpreter.CodeStack.IsEmpty);
            Assert.IsTrue(interpreter.ExecStack.IsEmpty);
            Assert.IsTrue(interpreter.NameStack.IsEmpty);
            Assert.IsTrue(interpreter.BooleanStack.IsEmpty);
        }
    }
}
