﻿using HeuristicLab.Algorithms.PushGP.Expressions;
using HeuristicLab.Algorithms.PushGP.Stack;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace HeuristicLab.Tests.Interpreter.Expressions
{
    [TestClass]
    public class IntegerExpressionTests : CommonTests<long>
    {
        protected override string TypeName { get { return "INTEGER"; } }
        protected override IStack<long> Stack { get { return interpreter.IntegerStack; } }

        [TestMethod]
        public void TestAdd()
        {
            interpreter.IntegerStack.Push(5, 5);
            interpreter.Interpret(new IntegerAddExpression());

            Assert.AreEqual(10, interpreter.IntegerStack.Top);

            this.TestStackCounts(integerStack: 1);
        }

        [TestMethod]
        public void TestAddWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("+", 1);
        }

        [TestMethod]
        public void TestSubtract()
        {
            interpreter.IntegerStack.Push(10, 5);
            interpreter.Interpret(new IntegerSubtractExpression());

            Assert.AreEqual(5, interpreter.IntegerStack.Top);

            this.TestStackCounts(integerStack: 1);
        }

        [TestMethod]
        public void TestSubtractWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("-", 1);
        }

        [TestMethod]
        public void TestMultiply()
        {
            interpreter.IntegerStack.Push(10, 5);
            interpreter.Interpret(new IntegerMultiplyExpression());

            Assert.AreEqual(50, interpreter.IntegerStack.Top);

            this.TestStackCounts(integerStack: 1);
        }

        [TestMethod]
        public void TestMultiplyWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("*", 1);
        }

        [TestMethod]
        public void TestDivide()
        {
            interpreter.IntegerStack.Push(10, 5);
            interpreter.Interpret(new IntegerDivideExpression());

            Assert.AreEqual(2, interpreter.IntegerStack.Top);

            this.TestStackCounts(integerStack: 1);
        }

        [TestMethod]
        public void TestDivideWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("/", 1);
        }

        [TestMethod]
        public void TestModulo()
        {
            interpreter.IntegerStack.Push(10, 5);
            interpreter.Interpret(new IntegerModuloExpression());

            Assert.AreEqual(0, interpreter.IntegerStack.Top);

            this.TestStackCounts(integerStack: 1);
        }

        [TestMethod]
        public void TestModuloWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("%", 1);
        }

        [TestMethod]
        public void TestMin()
        {
            interpreter.IntegerStack.Push(10, 5);
            interpreter.Interpret(new IntegerMinExpression());

            Assert.AreEqual(5, interpreter.IntegerStack.Top);

            this.TestStackCounts(integerStack: 1);
        }

        [TestMethod]
        public void TestMinWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("MIN", 1);
        }

        [TestMethod]
        public void TestMax()
        {
            interpreter.IntegerStack.Push(10, 5);
            interpreter.Interpret(new IntegerMaxExpression());

            Assert.AreEqual(10, interpreter.IntegerStack.Top);

            this.TestStackCounts(integerStack: 1);
        }

        [TestMethod]
        public void TestMaxWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("MAX", 1);
        }

        [TestMethod]
        public void TestSmallerThan()
        {
            interpreter.IntegerStack.Push(10, 5);
            interpreter.Interpret(new IntegerSmallerThanExpression());

            Assert.AreEqual(false, interpreter.BooleanStack.Top);

            this.TestStackCounts(booleanStack: 1);
        }

        [TestMethod]
        public void TestSmallerThanWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("<", 1);
        }

        [TestMethod]
        public void TestGreaterThan()
        {
            interpreter.IntegerStack.Push(10, 5);
            interpreter.Interpret(new IntegerGreaterThanExpression());

            Assert.AreEqual(true, interpreter.BooleanStack.Top);

            this.TestStackCounts(booleanStack: 1);
        }

        [TestMethod]
        public void TestGreaterThanWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments(">", 1);
        }

        [TestMethod]
        public void TestFromBooleanTrue()
        {
            interpreter.BooleanStack.Push(true);
            interpreter.Interpret(new IntegerFromBooleanExpression());

            Assert.AreEqual(1, interpreter.IntegerStack.Top);

            this.TestStackCounts(integerStack: 1);
        }

        [TestMethod]
        public void TestFromBooleanWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("FROMBOOLEAN");
        }

        [TestMethod]
        public void TestFromBooleanFalse()
        {
            interpreter.BooleanStack.Push(false);
            interpreter.Interpret(new IntegerFromBooleanExpression());

            Assert.AreEqual(0, interpreter.IntegerStack.Top);

            this.TestStackCounts(integerStack: 1);
        }

        [TestMethod]
        public void TestFromFloat()
        {
            interpreter.FloatStack.Push(1.5);
            interpreter.Interpret(new IntegerFromFloatExpression());

            Assert.AreEqual(1, interpreter.IntegerStack.Top);

            this.TestStackCounts(integerStack: 1);
        }

        [TestMethod]
        public void TestFromFloatWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("FROMFLOAT");
        }

        protected override long[] GetValues(int count)
        {
            var values = new long[count];

            for (long i = 0; i < count; i++)
            {
                values[i] = i;
            }

            return values;
        }

        protected override void CheckOtherStacksAreEmpty()
        {
            this.TestStackCounts(integerStack: null);
        }
    }
}
