﻿using HeuristicLab.Algorithms.PushGP.Expressions;
using HeuristicLab.Algorithms.PushGP.Generators;
using HeuristicLab.Algorithms.PushGP.Stack;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace HeuristicLab.Tests.Interpreter.Expressions
{
    [TestClass]
    public class NameExpressionTests : CommonTests<string>
    {

        protected override string TypeName { get { return "NAME"; } }
        protected override IStack<string> Stack { get { return interpreter.NameStack; } }

        [TestMethod]
        public void TestRand()
        {
            interpreter.Interpret(new NameRandExpression());

            Assert.IsTrue(interpreter.NameStack.Count == 1);

            this.CheckOtherStacksAreEmpty();
        }

        [TestMethod]
        public void TestRandomBound()
        {
            interpreter.CustomExpressions.Add("c1", new CodeNoopExpression());
            interpreter.CustomExpressions.Add("c2", new CodeNoopExpression());
            interpreter.CustomExpressions.Add("c3", new CodeNoopExpression());

            interpreter.Interpret(new NameRandBoundNameExpression());

            Assert.IsTrue(interpreter.CustomExpressions.Keys.Contains(interpreter.NameStack.Top));

            this.CheckOtherStacksAreEmpty();
        }

        [TestMethod]
        public void TestRandomBoundWithInsufficientArguments()
        {
            this.TestWithInsufficientArguments("RANDBOUNDNAME");
        }

        [TestMethod]
        public void TestQuote()
        {
            interpreter.Interpret(new NameQuoteExpression());

            Assert.IsTrue(interpreter.IsNameQuoteFlagSet);

            this.TestStackCounts();
        }

        protected override string[] GetValues(int count)
        {
            var ng = new NameGenerator();
            var values = new string[count];

            for (var i = 0; i < count; i++)
            {
                values[i] = ng.GetNextString();
            }

            return values;
        }

        protected override void CheckOtherStacksAreEmpty()
        {
            this.TestStackCounts(nameStack: null);
        }
    }
}
